@@ -58,13 +58,34 @@ double sqsum(double *X, int n, int j) {
 }
 
 double lasso(double z, double l1, double l2, double v) {
-  double s=0;
+  double s = 0;
   if (z > 0) s = 1;
   else if (z < 0) s = -1;
   if (fabs(z) <= l1) return(0);
   else return(s*(fabs(z)-l1)/(v*(1+l2)));
 }
 
+// MCP taken from ncvreg::ncvreg_init.c
+double MCP(double z, double l1, double l2, double gamma, double v) {
+  double s = 0;
+  if (z > 0) s = 1;
+  else if (z < 0) s = -1;
+  if (fabs(z) <= l1) return(0);
+  else if (fabs(z) <= gamma*l1*(1+l2)) return(s*(fabs(z)-l1)/(v*(1+l2-1/gamma)));
+  else return(z/(v*(1+l2)));
+}
+
+// SCAD taken from ncvreg::ncvreg_init.c
+double SCAD(double z, double l1, double l2, double gamma, double v) {
+  double s=0;
+  if (z > 0) s = 1;
+  else if (z < 0) s = -1;
+  if (fabs(z) <= l1) return(0);
+  else if (fabs(z) <= (l1*(1+l2)+l1)) return(s*(fabs(z)-l1)/(v*(1+l2)));
+  else if (fabs(z) <= gamma*l1*(1+l2)) return(s*(fabs(z)-gamma*l1/(gamma-1))/(v*(1-1/(gamma-1)+l2)));
+  else return(z/(v*(1+l2)));
+}
+
 // Gaussian loss
 double gLoss(double *r, int n) {
   double l = 0;
@@ -107,6 +128,42 @@ double crossprod_bm(XPtr<BigMatrix> xpMat, double *y_, int *row_idx_, double cen
   return sum;
 }
 
+// crossprod - cross product of y with jth column of a *standardized* X
+double crossprod_bm_no_std(XPtr<BigMatrix> xpMat, double *y_, int n_row, int j) {
+  MatrixAccessor<double> xAcc(*xpMat); // Initialize MatrixAccessor with the file-backed matrix
+  double *xCol = xAcc[j];  
+  double sum = 0.0;
+
+  for (int i = 0; i < n_row; i++) {
+    sum = sum + xCol[i] * y_[i];
+  }
+  
+  return sum;
+}
+
+
+
+// cumulative difference (to use in residual calculation; see ncvreg::rawfit_gaussian
+// lines 63 - 73 for reference)
+// TODO: revisit this and make gaussian_simple setup easier to read...
+// double cumdiff_bm(XPtr<BigMatrix> xpMat, NumericVector &a_, NumericVector &y_,
+//                    int n, int p){
+//   double *res = R_Calloc(n, double);
+//   MatrixAccessor<double> xAcc(*xpMat);
+//   
+//   for (int j=0; j<p; j++) {
+//     double *xCol = xAcc[j];
+//     for (int i=0; i < n; i++){
+//       res[i] -= xCol[i] * a_[i];
+//       
+//     }
+//   }
+//   
+//   return res;
+// }
+
+
+
 // crossprod of columns X_j and X_k
 double crossprod_bm_Xj_Xk(XPtr<BigMatrix> xMat, int *row_idx,
                           NumericVector &center, NumericVector &scale,
@@ -115,12 +172,12 @@ double crossprod_bm_Xj_Xk(XPtr<BigMatrix> xMat, int *row_idx,
   double *xCol_j = xAcc[j];
   double *xCol_k = xAcc[k];
   double sum_xj_xk = 0.0;
-  double res = 0.0;
+  // double res = 0.0;
   
   for (int i = 0; i < n; i++) {
     sum_xj_xk += xCol_j[row_idx[i]] * xCol_k[row_idx[i]];
   }
-  res = (sum_xj_xk - n * center[j] * center[k]) / (scale[j] * scale[k]);
+  double res = (sum_xj_xk - n * center[j] * center[k]) / (scale[j] * scale[k]);
   
   return res;
 }
@@ -139,7 +196,22 @@ double crossprod_resid(XPtr<BigMatrix> xpMat, double *y_, double sumY_, int *row
   return sum;
 }
 
-// update residul vector
+//crossprod_resid - given specific rows of *standardized* X: separate computation
+// double crossprod_resid_no_std(XPtr<BigMatrix> xpMat, double *y_, double sumY_,
+//                         int n_row, int j) {
+// 
+//   MatrixAccessor<double> xAcc(*xpMat);
+//   double *xCol = xAcc[j];
+//   
+//   double sum = 0.0;
+//   for (int i=0; i < n_row; i++) {
+//     sum = sum + xCol[i] * y_[i];
+//   }
+//   sum = (sum * sumY_);
+//   return sum;
+// }
+
+// update residual vector
 void update_resid(XPtr<BigMatrix> xpMat, double *r, double shift, int *row_idx_, 
                   double center_, double scale_, int n_row, int j) {
   MatrixAccessor<double> xAcc(*xpMat);
@@ -149,7 +221,19 @@ void update_resid(XPtr<BigMatrix> xpMat, double *r, double shift, int *row_idx_,
   }
 }
 
-// update residul vector and eta vector
+// update residual vector -- no standardization
+void update_resid_no_std(XPtr<BigMatrix> xpMat, double *r, double shift,
+                  int n_row, int j) {
+
+  MatrixAccessor<double> xAcc(*xpMat);
+  double *xCol = xAcc[j];
+  for (int i=0; i < n_row; i++) {
+    r[i] -= shift * xCol[i];
+  }
+}
+
+
+// update residual vector and eta vector
 void update_resid_eta(double *r, double *eta, XPtr<BigMatrix> xpMat, double shift, 
                       int *row_idx_, double center_, double scale_, int n, int j) {
   
@@ -164,13 +248,11 @@ void update_resid_eta(double *r, double *eta, XPtr<BigMatrix> xpMat, double shif
 }
 
 // Sum of squares of jth column of X
-double sqsum_bm(SEXP xP, int n_row, int j, int useCores) {
-  XPtr<BigMatrix> xpMat(xP); //convert to big.matrix pointer;
-  MatrixAccessor<double> xAcc(*xpMat);
-  double *xCol = xAcc[j];
+double sqsum_bm(XPtr<BigMatrix> xpMat, int n_row, int j) {
+  MatrixAccessor<double> xAcc(*xpMat); // Initialize MatrixAccessor with the file-backed matrix
   
+  double *xCol = xAcc[j];
   double val = 0.0;
-  // #pragma omp parallel for reduction(+:val)
   for (int i=0; i < n_row; i++) {
     val += pow(xCol[i], 2);
   }
@@ -222,7 +304,7 @@ double wsqsum_bm(XPtr<BigMatrix> xpMat, double *w, int *row_idx_, double center_
   return val;
 }
 
-// standardize
+// standardize & get residual 
 void standardize_and_get_residual(NumericVector &center, NumericVector &scale, 
                                   int *p_keep_ptr, vector<int> &col_idx, //columns to keep, removing columns whose scale < 1e-6
                                   vector<double> &z, double *lambda_max_ptr,
@@ -264,6 +346,40 @@ void standardize_and_get_residual(NumericVector &center, NumericVector &scale,
   *lambda_max_ptr = zmax / alpha;
 }
 
+// // get residual only -- need this in gaussian_simple 
+// void get_residual(vector<double> &z, 
+//                   double lambda, 
+//                   int *xmax_ptr,
+//                   XPtr<BigMatrix> xMat, 
+//                   double *y, 
+//                   double alpha,
+//                   int n, 
+//                   int p) {
+//   MatrixAccessor<double> xAcc(*xMat);
+//   double *xCol;
+//   double sum;
+//   double zmax = 0.0, zj = 0.0;
+//   int i, j;
+//   
+//   for (j = 0; j < p; j++) {
+//     xCol = xAcc[j];
+//     sum = 0.0;
+//     
+//     for (i = 0; i < n; i++) {
+//       // NB: this assumes X matrix has *already* been standardized 
+//       sum = sum + xCol[i] * y[i];
+//     }
+//     
+//     zj = (sum * sum) / (n); //residual
+//     if (fabs(zj) > zmax) {
+//       zmax = fabs(zj);
+//       *xmax_ptr = j; // xmax_ptr is the index in the raw xMat, not index in col_idx!
+//     }
+//     z.push_back(zj);
+//   }
+//   lambda = zmax / alpha;
+// }
+
 // check KKT conditions over features in the inactive set
 int check_inactive_set(int *e1, vector<double> &z, XPtr<BigMatrix> xpMat, int *row_idx, 
                        vector<int> &col_idx, NumericVector &center, NumericVector &scale, double *a,
@@ -356,6 +472,40 @@ int check_rest_safe_set(int *ever_active, int *strong_set, int *discard_beta, ve
   return violations;
 }
 
+// check KKT conditions over features in (the safe set - the strong set)
+int check_rest_safe_set_no_std(int *ever_active, int *strong_set, int *discard_beta, vector<double> &z,
+                        XPtr<BigMatrix> xpMat, int *row_idx, vector<int> &col_idx,
+                        double *a, double lambda,
+                        double sumResid, double alpha, double *r, double *m, int n, int p) {
+  if (!xpMat) return 0; // check 
+  MatrixAccessor<double> xAcc(*xpMat);
+  double *xCol;
+  double sum = 0.0;
+  double l1 = 0.0;
+  double l2 = 0.0;
+  int j, jj, violations = 0;
+#pragma omp parallel for private(j, sum, l1, l2) reduction(+:violations) schedule(static) 
+  for (j = 0; j < p; j++) {
+    if (strong_set[j] == 0 && discard_beta[j] == 0) {
+      jj = col_idx[j];
+      xCol = xAcc[jj];
+      sum = 0.0;
+      for (int i=0; i < n; i++) {
+        sum = sum + xCol[row_idx[i]] * r[i];
+      }
+      z[j] = (sum  * sumResid) / (n);
+      
+      l1 = lambda * m[jj] * alpha;
+      l2 = lambda * m[jj] * (1 - alpha);
+      if (fabs(z[j] - a[j] * l2) > l1) {
+        ever_active[j] = strong_set[j] = 1;
+        violations++;
+      }
+    }
+  }
+  return violations;
+}
+
 // check KKT conditions over features in the strong set
 int check_strong_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, 
                      int *row_idx, vector<int> &col_idx,
@@ -388,6 +538,42 @@ int check_strong_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat,
   return violations;
 }
 
+// check KKT conditions over features in the strong set on *standardized* X 
+int check_strong_set_no_std(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, 
+                     int *row_idx, vector<int> &col_idx, double *a,
+                     double lambda, double sumResid, double alpha, 
+                     double *r, double *m, int n, int p) {
+  MatrixAccessor<double> xAcc(*xpMat);
+  double *xCol;
+  double sum = 0.0;
+  double l1 = 0.0;
+  double l2 = 0.0;
+  int j, jj, violations = 0;
+  
+#pragma omp parallel for private(j, sum, l1, l2) reduction(+:violations) schedule(static) 
+  for (j = 0; j < p; j++) {
+    if (e1[j] == 0 && e2[j] == 1) {
+      jj = col_idx[j];
+      xCol = xAcc[jj];
+      sum = 0.0;
+      for (int i=0; i < n; i++) {
+        sum = sum + xCol[row_idx[i]] * r[i];
+      }
+      z[j] = (sum * sumResid) / (n);
+      
+      l1 = lambda * m[jj] * alpha;
+      l2 = lambda * m[jj] * (1 - alpha);
+      if(fabs(z[j] - a[j] * l2) > l1) {
+        e1[j] = 1;
+        violations++;
+      }
+    }
+  }
+  return violations;
+}
+
+
+
 // check KKT conditions over features in the rest set
 int check_rest_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, int *row_idx, 
                    vector<int> &col_idx, NumericVector &center, NumericVector &scale, double *a,
