@@ -34,6 +34,8 @@ double sum(double *x, int n);
 
 // Sum of squares of jth column of X
 double sqsum(double *X, int n, int j);
+// sqsum for file-backed X
+double sqsum_bm(XPtr<BigMatrix> xpMat, int n_row, int j);
  
 double crossprod(double *X, double *y, int n, int j);
 
@@ -41,6 +43,10 @@ int sum(int *vec, int p);
 
 double lasso(double z, double l1, double l2, double v);
 
+double MCP(double z, double l1, double l2, double gamma, double v);
+
+double SCAD(double z, double l1, double l2, double gamma, double v);
+
 double gLoss(double *r, int n);
 
 // get X[i, j]: i-th row, j-th column element
@@ -53,6 +59,9 @@ double get_elem_bm(XPtr<BigMatrix> xpMat, double center_, double scale_, int i,
 double crossprod_bm(XPtr<BigMatrix> xpMat, double *y_, int *row_idx_, double center_, 
                     double scale_, int n_row, int j);
 
+//crossprod - for gaussian_simple()
+double crossprod_bm_no_std(XPtr<BigMatrix> xpMat, double *y_, int n_row, int j);
+
 // crossprod of columns X_j and X_k
 double crossprod_bm_Xj_Xk(XPtr<BigMatrix> xMat, int *row_idx,
                           NumericVector &center, NumericVector &scale,
@@ -65,10 +74,19 @@ double crossprod_bm_Xj_Xk(XPtr<BigMatrix> xMat, int *row_idx,
 double crossprod_resid(XPtr<BigMatrix> xpMat, double *y_, double sumY_, int *row_idx_, 
                        double center_, double scale_, int n_row, int j);
 
+//crossprod_resid - given specific rows of *standardized* X: separate computation
+// double crossprod_resid_no_std(XPtr<BigMatrix> xpMat, double *y_, double sumY_,
+//                               int n_row, int j);
+
 // update residul vector if variable j enters eligible set
 void update_resid(XPtr<BigMatrix> xpMat, double *r, double shift, int *row_idx_, 
                   double center_, double scale_, int n_row, int j);
 
+
+// update residual vector if variable j enters eligible set; NO standardization
+void update_resid_no_std(XPtr<BigMatrix> xpMat, double *r, double shift,
+                   int n_row, int j);
+
 // update residul vector and eta vector
 void update_resid_eta(double *r, double *eta, XPtr<BigMatrix> xpMat, double shift, 
                       int *row_idx_, double center_, double scale_, int n, int j);
@@ -96,6 +114,11 @@ void standardize_and_get_residual(NumericVector &center, NumericVector &scale,
                                   int *xmax_ptr, XPtr<BigMatrix> xMat, double *y, 
                                   int *row_idx, double alpha, int n, int p);
 
+// get residuals 
+// void get_residual(vector<double> &z, double lambda,
+//                   int *xmax_ptr, XPtr<BigMatrix> xMat, double *y, 
+//                   double alpha, int n, int p);
+
 // check KKT conditions over features in the inactive set
 int check_inactive_set(int *e1, vector<double> &z, XPtr<BigMatrix> xpMat, int *row_idx, 
                        vector<int> &col_idx, NumericVector &center, NumericVector &scale, double *a,
@@ -114,6 +137,12 @@ int check_rest_safe_set(int *ever_active, int *strong_set, int *discard_beta, ve
                         NumericVector &center, NumericVector &scale, double *a, double lambda,
                         double sumResid, double alpha, double *r, double *m, int n, int p);
 
+// check KKT conditions over features in (the safe set - the strong set) for *standardized* X
+int check_rest_safe_set_no_std(int *ever_active, int *strong_set, int *discard_beta, vector<double> &z,
+                        XPtr<BigMatrix> xpMat, int *row_idx, vector<int> &col_idx,
+                        double *a, double lambda,
+                        double sumResid, double alpha, double *r, double *m, int n, int p);
+
 // check KKT conditions over features in the strong set
 int check_strong_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, 
                      int *row_idx, vector<int> &col_idx,
@@ -121,6 +150,12 @@ int check_strong_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat,
                      double lambda, double sumResid, double alpha, 
                      double *r, double *m, int n, int p);
 
+// check KKT conditions over features in the strong set for *standardized* X
+int check_strong_set_no_std(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, 
+                     int *row_idx, vector<int> &col_idx, double *a,
+                     double lambda, double sumResid, double alpha, 
+                     double *r, double *m, int n, int p);
+
 // check KKT conditions over features in the rest set
 int check_rest_set(int *e1, int *e2, vector<double> &z, XPtr<BigMatrix> xpMat, int *row_idx, 
                    vector<int> &col_idx, NumericVector &center, NumericVector &scale, double *a,
